/******************************************************************************
* Copyright (c) 2011, Michael P. Gerlek (mpg@flaxen.com)
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following
* conditions are met:
*
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in
*       the documentation and/or other materials provided
*       with the distribution.
*     * Neither the name of Hobu, Inc. or Flaxen Geo Consulting nor the
*       names of its contributors may be used to endorse or promote
*       products derived from this software without specific prior
*       written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
* FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
* COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
* AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
* OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
* OF SUCH DAMAGE.
****************************************************************************/

#ifndef INCLUDED_PIPELINEWRITER_HPP
#define INCLUDED_PIPELINEWRITER_HPP

#include <pdal/pdal_internal.hpp>
#include <pdal/Options.hpp>
#include <pdal/Metadata.hpp>

#include <string>

#include <boost/property_tree/ptree.hpp>

namespace pdal
{

class PipelineManager;
class PointBuffer;

class PDAL_DLL PipelineWriter
{
public:
    PipelineWriter(const PipelineManager&);
    ~PipelineWriter();

    void writePipeline(const std::string& filename) const;
    inline void setPointBuffer(PointBuffer const* buffer) { m_buffer = buffer; }

    static void write_option_ptree(boost::property_tree::ptree& tree, const Options& opts);
    static void write_metadata_ptree(boost::property_tree::ptree& tree, const Metadata& mdata);
    static boost::property_tree::ptree get_metadata_entry(boost::property_tree::ptree const& input);
private:
    const PipelineManager& m_manager;
    
    // The user can set a PointBuffer on the PipelineWriter and
    // information from it will be written into the XML
    PointBuffer const* m_buffer;

    PipelineWriter& operator=(const PipelineWriter&); // not implemented
    PipelineWriter(const PipelineWriter&); // not implemented
};


} // namespace pdal

#endif
